@@ -4,22 +4,24 @@ function(sgp_object,
 		years=NULL,
 		content_areas=NULL,
 		grades=NULL,
+		exclude.years=NULL,
 		sgp.config=NULL,
 		sgp.baseline.config=NULL,
 		sgp.baseline.panel.years=NULL,
 		sgp.percentiles.baseline.max.order=3,
 		return.matrices.only=FALSE,
 		calculate.baseline.sgps=TRUE,
-		calculate.baseline.simex=NULL,
+		calculate.simex.baseline=NULL,
 		goodness.of.fit.print=TRUE,
 		parallel.config=NULL,
+		SGPt=NULL,
 		...) {
 
 
 	started.at <- proc.time()
 	message(paste("\tStarted baselineSGP", date(), "\n"))
 
-	VALID_CASE <- YEAR <- GRADE <- CONTENT_AREA <- YEAR_WITHIN <- NULL ### To prevent R CMD check warnings
+	VALID_CASE <- YEAR <- GRADE <- CONTENT_AREA <- YEAR_WITHIN <- COHORT_YEAR <- NULL ### To prevent R CMD check warnings
 
 	### Create state (if NULL) from sgp_object (if possible)
 
@@ -28,8 +30,8 @@ function(sgp_object,
 		state <- getStateAbbreviation(tmp.name, "baselineSGP")
 	}
 
-	if (identical(calculate.baseline.simex, TRUE)) {
-		calculate.baseline.simex <- list(state=state, lambda=seq(0,2,0.5), simulation.iterations=50, simex.sample.size=25000, extrapolation="linear", save.matrices=TRUE)
+	if (identical(calculate.simex.baseline, TRUE)) {
+		calculate.simex.baseline <- list(state=state, lambda=seq(0,2,0.5), simulation.iterations=75, simex.sample.size=5000, extrapolation="linear", save.matrices=TRUE)
 	}
 
 	### Syncronize "return.matrices.only" and "calculate.baseline.sgps" arguments
@@ -39,11 +41,23 @@ function(sgp_object,
 		calculate.baseline.sgps <- FALSE
 	}
 
-	if (!is.null(SGPstateData[[state]][["SGP_Configuration"]][["sgp.loss.hoss.adjustment"]])) {
-		sgp.loss.hoss.adjustment <- SGPstateData[[state]][["SGP_Configuration"]][["sgp.loss.hoss.adjustment"]]
+	if (!is.null(SGP::SGPstateData[[state]][["SGP_Configuration"]][["sgp.loss.hoss.adjustment"]])) {
+		sgp.loss.hoss.adjustment <- SGP::SGPstateData[[state]][["SGP_Configuration"]][["sgp.loss.hoss.adjustment"]]
 	} else {
 		sgp.loss.hoss.adjustment <- NULL
 	}
+	
+	if (!is.null(SGP::SGPstateData[[state]][["SGP_Configuration"]][["sgp.minimum.default.panel.years"]])) {
+		sgp.minimum.default.panel.years <- SGP::SGPstateData[[state]][["SGP_Configuration"]][["sgp.minimum.default.panel.years"]]
+	} else sgp.minimum.default.panel.years <- 3
+
+	if (!is.null(SGPt)) {
+		if (identical(SGPt, TRUE)) SGPt <- "DATE"
+		if (!all(SGPt %in% names(sgp_object@Data))) {
+			tmp.messages <- c(tmp.messages, "\t\tNOTE: Variables", paste(SGPt, collapse=", "), "are not all contained in the supplied 'sgp_object@Data'. 'SGPt' is set to NULL.\n")
+			SGPt <- NULL
+		}
+	}
 
 
 	############################################
@@ -53,7 +67,8 @@ function(sgp_object,
 	############################################
 
 
-	baselineSGP_Internal <- function(sgp_object, state, years, content_areas, grade.sequences, baseline.grade.sequences.lags, knots.boundaries.iter, parallel.config, use.my.coefficient.matrices, calculate.simex) {
+	baselineSGP_Internal <- function(sgp_object, state, years, content_areas, grade.sequences, baseline.grade.sequences.lags, 
+		knots.boundaries.iter, parallel.config, use.my.coefficient.matrices, calculate.simex) {
 
 		started.at <- proc.time()
 		started.date <- date()
@@ -82,34 +97,36 @@ function(sgp_object,
 			setkey(tmp_sgp_data_for_analysis, VALID_CASE, CONTENT_AREA, YEAR, GRADE)
 		}
 		tmp.year.sequence <- test.year.sequence(content_areas, years, grade.sequences, baseline.grade.sequences.lags)
+		if (!is.null(exclude.years)) {
+			tmp.year.sequence <- tmp.year.sequence[sapply(tmp.year.sequence, function(x) !tail(x, 1) %in% exclude.years)]
+		}
 		tmp.list <- list()
 		for (k in seq_along(tmp.year.sequence)) {
 			tmp.sgp.iter <- sgp.baseline.config[[iter]] # Convert sgp.baseline.config into a valid sgp.iter for getPanelData
 			names(tmp.sgp.iter) <- gsub('sgp.baseline.', 'sgp.', names(tmp.sgp.iter))
 			tmp.sgp.iter$sgp.panel.years <- tmp.year.sequence[[k]]
 			tmp.sgp.iter$sgp.grade.sequences <- tmp.sgp.iter$sgp.grade.sequences
+			if (!is.null(tmp.sgp.iter$sgp.exclude.sequences)) tmp.sgp.iter$sgp.exclude.sequences <- tmp.sgp.iter$sgp.exclude.sequences[COHORT_YEAR %in% tail(tmp.sgp.iter$sgp.panel.years, 1)]
 			tmp.list[[k]] <- getPanelData(tmp_sgp_data_for_analysis, "sgp.percentiles", sgp.iter = tmp.sgp.iter)[,
-				getPanelDataVnames("sgp.percentiles", tmp.sgp.iter, names(tmp_sgp_data_for_analysis))]
-			names(tmp.list[[k]]) <- c("ID", paste("GRADE", rev(seq_along(tmp.year.sequence[[k]])), sep="_"), 
-				paste("SCALE_SCORE", rev(seq_along(tmp.year.sequence[[k]])), sep="_")) # Use rev() in case vars are added: 1= current, 2= first prior, etc.
+				getPanelDataVnames("sgp.percentiles", tmp.sgp.iter, names(tmp_sgp_data_for_analysis)), with=FALSE]
+			setnames(tmp.list[[k]], c("ID", paste("GRADE", rev(seq_along(tmp.year.sequence[[k]])), sep="_"), paste("SCALE_SCORE", rev(seq_along(tmp.year.sequence[[k]])), sep="_")))
 		}
-		tmp.df <- rbind.fill(tmp.list)
-
+		tmp.dt <- rbindlist(tmp.list, fill=TRUE)
 
 		### Calculate Coefficient Matrices and return list containing coefficient matrices
 		
-		if (!is.null(calculate.baseline.simex)) TMP_Coefficient_Matrices = sgp_object@SGP[["Coefficient_Matrices"]] else TMP_Coefficient_Matrices <- list()
+		if (!is.null(calculate.simex)) TMP_Coefficient_Matrices = sgp_object@SGP[["Coefficient_Matrices"]] else TMP_Coefficient_Matrices <- list()
 
 		tmp_sgp_list <- list(Coefficient_Matrices =
 			studentGrowthPercentiles(
-				panel.data=list(Panel_Data= tmp.df, Coefficient_Matrices = TMP_Coefficient_Matrices, #  Add Coef Matrices for SIMEX
-					Knots_Boundaries=getKnotsBoundaries(knots.boundaries.iter, state, c("Baseline", "sgp.percentiles"))),
+				panel.data=list(Panel_Data=tmp.dt, Coefficient_Matrices=TMP_Coefficient_Matrices, # Add Coef Matrices for SIMEX
+					Knots_Boundaries=getKnotsBoundaries(knots.boundaries.iter, state, "sgp.percentiles.baseline", "BASELINE")),
 				sgp.labels=list(my.year="BASELINE", my.subject=tail(content_areas, 1)),
 				use.my.knots.boundaries=list(my.year="BASELINE", my.subject=tail(content_areas, 1)),
 				use.my.coefficient.matrices= use.my.coefficient.matrices,
 				calculate.sgps=FALSE,
 				goodness.of.fit=FALSE,
-				drop.nonsequential.grade.progression.variables=FALSE, # taken care of in data reshape above.
+				drop.nonsequential.grade.progression.variables=FALSE,
 				grade.progression=grade.sequences,
 				content_area.progression=content_areas,
 				year.progression=rep("BASELINE", length(content_areas)),
@@ -118,11 +135,12 @@ function(sgp_object,
 				print.time.taken=FALSE,
 				parallel.config=parallel.config,
 				calculate.simex=calculate.simex,
+				SGPt=SGPt,
 				...)[["Coefficient_Matrices"]])
 
 		message(paste("\tStarted baselineSGP Coefficient Matrix Calculation:", started.date))
 		message(paste("\tContent Area: ", tail(content_areas, 1), ", Grade Progression: ", paste(grade.sequences, collapse=", "), ". ", sep=""))
-		message(paste("\tFinished baselineSGP Coefficient Matrix Calculation ", date(), " in ", timetaken(started.at), ".\n", sep=""))
+		message(paste("\tFinished baselineSGP Coefficient Matrix Calculation ", date(), " in ", convertTime(timetaken(started.at)), ".\n", sep=""))
 
 		return(tmp_sgp_list)
 
@@ -151,10 +169,11 @@ function(sgp_object,
 	###
 	#################################################################################
 
-	if (is.null(SGPstateData[[state]][["Baseline_splineMatrix"]])) {
+	if (is.null(SGP::SGPstateData[[state]][["Baseline_splineMatrix"]])) {
 		
 		if (is.null(sgp.baseline.config)) {
-			sgp.baseline.config <- getSGPBaselineConfig(sgp_object, content_areas, grades, sgp.baseline.panel.years)
+			sgp.baseline.config <- getSGPBaselineConfig(sgp_object, content_areas, grades, sgp.baseline.panel.years, 
+				sgp.percentiles.baseline.max.order = sgp.percentiles.baseline.max.order, calculate.simex.baseline = calculate.simex.baseline)
 		} else {
 			sgp.baseline.config <- checkConfig(sgp.baseline.config, "Baseline")
 		}
@@ -177,7 +196,7 @@ function(sgp_object,
 
 		sgp_object@SGP <- mergeSGP(Reduce(mergeSGP, tmp.list), sgp_object@SGP)
 	} else {
-		sgp_object@SGP <- mergeSGP(sgp_object@SGP, SGPstateData[[state]][["Baseline_splineMatrix"]])
+		sgp_object@SGP <- mergeSGP(sgp_object@SGP, SGP::SGPstateData[[state]][["Baseline_splineMatrix"]])
 	}
 
 
@@ -187,9 +206,10 @@ function(sgp_object,
 	###
 	#################################################################################
 
-	if (!is.null(calculate.baseline.simex)) {
+	if (!is.null(calculate.simex.baseline)) {
 		if (is.null(sgp.baseline.config)) {
-			sgp.baseline.config <- getSGPBaselineConfig(sgp_object, content_areas, grades, sgp.baseline.panel.years)
+			sgp.baseline.config <- getSGPBaselineConfig(sgp_object, content_areas, grades, sgp.baseline.panel.years,
+				sgp.percentiles.baseline.max.order = sgp.percentiles.baseline.max.order, calculate.simex.baseline = calculate.simex.baseline)
 		} else {
 			sgp.baseline.config <- checkConfig(sgp.baseline.config, "Baseline")
 		}
@@ -207,7 +227,7 @@ function(sgp_object,
 					knots.boundaries.iter=sgp.baseline.config[[iter]],
 					parallel.config=parallel.config,
 					use.my.coefficient.matrices=list(my.year="BASELINE", my.subject=tail(sgp.baseline.config[[iter]][["sgp.baseline.content.areas"]], 1)),
-					calculate.simex=calculate.baseline.simex)
+					calculate.simex=calculate.simex.baseline)
 		}
 
 		sgp_object@SGP <- mergeSGP(Reduce(mergeSGP, tmp.list), sgp_object@SGP)
@@ -233,16 +253,17 @@ function(sgp_object,
 			setkey(tmp_sgp_data_for_analysis, VALID_CASE, CONTENT_AREA, YEAR, GRADE)
 		}
 
-		sgp.config <- getSGPConfig(sgp_object, tmp_sgp_object, content_areas, years, grades, sgp.config,
+		par.sgp.config <- getSGPConfig(sgp_object, state, tmp_sgp_object, content_areas, years, grades, sgp.config, trim.sgp.config=TRUE,
 			sgp.percentiles=FALSE, sgp.projections=FALSE, sgp.projections.lagged=FALSE,
 			sgp.percentiles.baseline=TRUE, sgp.projections.baseline=FALSE, sgp.projections.lagged.baseline=FALSE,
-			sgp.config.drop.nonsequential.grade.progression.variables=TRUE)
-		sgp.config.baseline <- sgp.config[which(sapply(sgp.config, function(x) !identical(x[['base.gp']], "NO_BASELINE_COEFFICIENT_MATRICES")))]
+			sgp.config.drop.nonsequential.grade.progression.variables=TRUE, sgp.minimum.default.panel.years=sgp.minimum.default.panel.years,
+			sgp.projections.max.forward.progression.years=NULL, sgp.use.my.coefficient.matrices=NULL, calculate.simex.baseline=calculate.simex.baseline, 
+			SGPt=SGPt)
 
-		for (sgp.iter in sgp.config.baseline) {
+		for (sgp.iter in par.sgp.config[['sgp.percentiles.baseline']]) {
 
 			panel.data=within(tmp_sgp_object, assign("Panel_Data", getPanelData(tmp_sgp_data_for_analysis, "sgp.percentiles", sgp.iter)))
-			tmp.knots.boundaries <- getKnotsBoundaries(sgp.iter, state, c("Standard", "sgp.percentiles")) # Get specific knots and boundaries in case course sequence is different
+			tmp.knots.boundaries <- getKnotsBoundaries(sgp.iter, state, "sgp.percentiles") # Get specific knots and boundaries in case course sequence is different
 			panel.data[["Knots_Boundaries"]][[names(tmp.knots.boundaries)]] <- tmp.knots.boundaries[[names(tmp.knots.boundaries)]]
 
 			tmp_sgp_object <- studentGrowthPercentiles(
@@ -252,14 +273,16 @@ function(sgp_object,
 					use.my.knots.boundaries=list(my.year=tail(sgp.iter[['sgp.panel.years']], 1), my.subject=tail(sgp.iter[['sgp.content.areas']], 1)),
 					use.my.coefficient.matrices=list(my.year="BASELINE", my.subject=tail(sgp.iter[['sgp.content.areas']], 1)),
 					growth.levels=state,
-					panel.data.vnames=getPanelDataVnames("sgp.percentiles", sgp.iter),
-					grade.progression=sgp.iter[['base.gp']],
-					content_area.progression=tail(sgp.iter[['sgp.content.areas']], min(sgp.iter[['max.order']], sgp.percentiles.baseline.max.order)+1),
-					num.prior=min(sgp.iter[['max.order']], sgp.percentiles.baseline.max.order),
-					percentile.cuts=SGPstateData[[state]][['SGP_Configuration']][['percentile.cuts']],
+					panel.data.vnames=getPanelDataVnames("sgp.percentiles", sgp.iter, names(tmp_sgp_data_for_analysis)),
+					grade.progression=sgp.iter[['sgp.grade.sequences']],
+					content_area.progression=tail(sgp.iter[['sgp.content.areas']], min(sgp.iter[['sgp.baseline.max.order']], sgp.percentiles.baseline.max.order)+1),
+					year.progression=rep("BASELINE", length(sgp.iter[['sgp.content.areas']])),
+					num.prior =min(sgp.iter[['sgp.baseline.max.order']], sgp.percentiles.baseline.max.order),
+					percentile.cuts=SGP::SGPstateData[[state]][['SGP_Configuration']][['percentile.cuts']],
 					drop.nonsequential.grade.progression.variables=FALSE,
 					exact.grade.progression.sequence=sgp.iter[['sgp.exact.grade.progression']],
 					sgp.loss.hoss.adjustment=sgp.loss.hoss.adjustment,
+					SGPt=SGPt,
 					...)
 
 		} ### END sgp.iter loop
@@ -277,16 +300,16 @@ function(sgp_object,
 	###
 	############################################################
 
-	message(paste("\tFinished baselineSGP", date(), "in", timetaken(started.at), "\n"))
+	message(paste("\tFinished baselineSGP", date(), "in", convertTime(timetaken(started.at)), "\n"))
 
 	if (return.matrices.only) {
 		tmp.list <- list()
-		if (is.null(SGPstateData[[state]][["Baseline_splineMatrix"]])) {
+		if (is.null(SGP::SGPstateData[[state]][["Baseline_splineMatrix"]])) {
 			for (ca in unique(sapply(sgp.baseline.config, function(x) tail(x[["sgp.baseline.content.areas"]],1)))) {
 				tmp.list[[paste(ca, ".BASELINE", sep="")]] <- sgp_object@SGP[["Coefficient_Matrices"]][[paste(ca, ".BASELINE", sep="")]]
 			}
 		}
-		if (!is.null(calculate.baseline.simex)) {
+		if (!is.null(calculate.simex.baseline)) {
 			for (ca in unique(sapply(sgp.baseline.config, function(x) tail(x[["sgp.baseline.content.areas"]],1)))) {
 				tmp.list[[paste(ca, ".BASELINE.SIMEX", sep="")]] <- sgp_object@SGP[["Coefficient_Matrices"]][[paste(ca, ".BASELINE.SIMEX", sep="")]]
 			}			
