@@ -57,7 +57,7 @@ function(sgp_object,
 		parallel.config=NULL) {
 
 	started.at.visualizeSGP <- proc.time()
-	message(paste("\nStarted visualizeSGP", date(), "\n"))
+	messageSGP(paste("\nStarted visualizeSGP", date(), "\n"))
 
 	### Setting variables to NULL to prevent R CMD check warnings
 
@@ -91,23 +91,6 @@ function(sgp_object,
 
 	pretty_year <- function(x) sub("_", "-", x)
 
-	get.max.order.for.progression <- function(year, content_area) {
-		if (is.null(gaPlot.max.order.for.progression)) {
-			if (is.null(SGP::SGPstateData[[state]][["Assessment_Program_Information"]][["Scale_Change"]][[content_area]])) {
-				return(NULL)
-			} else {
-				tmp <- as.numeric(tail(unlist(strsplit(as.character(year), "_")), 1)) -
-					as.numeric(tail(unlist(strsplit(as.character(SGP::SGPstateData[[state]][["Assessment_Program_Information"]][["Scale_Change"]][[content_area]]), "_")), 1))
-				if (tmp < 0) return(NULL)
-				if (tmp > 0) return(as.numeric(tmp))
-				if (tmp==0) message(paste("\tNOTE: Based upon state scale changes in ", pretty_year(year),
-					". student growth projections are not possible. No student growth projections will be generated.\n", sep=""))
-			}
-		} else {
-			return(gaPlot.max.order.for.progression)
-		}
-	} ### END get.max.order.for.progression
-
 	get.sgPlot.iter <- function(districts.and.schools) {
 		tmp.list <- list()
 		for (k in 1:dim(districts.and.schools)[1]) {
@@ -158,14 +141,13 @@ function(sgp_object,
 		tmp.dt <- rbindlist(tmp.dt.list, fill=TRUE)
 
 		if (!is.null(gaPlot.students)) {
-			 tmp.dt <- merge(tmp.dt, gaPlot.students)
-			 names(tmp.dt)[dim(tmp.dt)[2]] <- "ID"
+			tmp.dt <- data.table(ID=rep(gaPlot.students, each=dim(tmp.dt)[1]), tmp.dt)
 		}
 
 		for (i in seq(dim(tmp.dt)[1])) {
 			tmp.gaPlot.list[[i]] <- list(YEAR=tmp.dt[["YEAR"]][i], CONTENT_AREA=tmp.dt[["CONTENT_AREA"]][i], ID=tmp.dt[["ID"]][i], BASELINE=tmp.dt[["BASELINE"]][i])
 			if (identical(SGP::SGPstateData[[state]][["Assessment_Program_Information"]][["Assessment_Transition"]][["Year"]], tmp.dt[["YEAR"]][i])) {
-				tmp.gaPlot.list[[i]][['EQUATED']] <- sgp_object@SGP[['Linkages']]
+				tmp.gaPlot.list[[i]][['EQUATED']] <- list(State=state, Year=tmp.dt[["YEAR"]][i], Linkages=sgp_object@SGP[['Linkages']])
 			} else {
 				tmp.gaPlot.list[[i]][['EQUATED']] <- NULL
 			}
@@ -248,7 +230,7 @@ function(sgp_object,
 					growthAchievementPlot(
 						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
-						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
+						gaPlot.max.order.for.progression=gaPlot.max.order.for.progression,
 						gaPlot.start.points=gaPlot.start.points,
 						state=state,
 						content_area=gaPlot.iter[["CONTENT_AREA"]],
@@ -268,7 +250,7 @@ function(sgp_object,
 					growthAchievementPlot(
 						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
-						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
+						gaPlot.max.order.for.progression=gaPlot.max.order.for.progression,
 						gaPlot.start.points=gaPlot.start.points,
 						state=state,
 						content_area=gaPlot.iter[["CONTENT_AREA"]],
@@ -286,7 +268,7 @@ function(sgp_object,
 					growthAchievementPlot(
 						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
-						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
+						gaPlot.max.order.for.progression=gaPlot.max.order.for.progression,
 						gaPlot.start.points=gaPlot.start.points,
 						state=state,
 						content_area=gaPlot.iter[["CONTENT_AREA"]],
@@ -1130,10 +1112,10 @@ if (sgPlot.produce.plots) {
 	} # END else Parallel Processing
 } ## END if (sgPlot.produce.plots)
 
-	message(paste("Finished studentGrowthPlot in visualizeSGP", date(), "in", convertTime(timetaken(started.at)), "\n"))
+	messageSGP(paste("Finished studentGrowthPlot in visualizeSGP", date(), "in", convertTime(timetaken(started.at)), "\n"))
 
 } ## END if ("studentGrowthPlot" %in% plot.types)
 
-	message(paste("Finished visualizeSGP", date(), "in", convertTime(timetaken(started.at.visualizeSGP)), "\n"))
+	messageSGP(paste("Finished visualizeSGP", date(), "in", convertTime(timetaken(started.at.visualizeSGP)), "\n"))
 
 } ## END visualizeSGP Function
